/******** <lalVerbatim file="FCTInitSearchCV"> ******** 
Author: Wen, L. 
$Id: FCTInitSearch.c,v 1.7 2001/08/27 20:04:50 lwen Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTInitSearch.h> 

NRCSID( FCTINITSEARCHC, "$Id: FCTInitSearch.c,v 1.7 2001/08/27 20:04:50 lwen Exp $");


void LALInitSearch( 
		   LALStatus           *status, 
		   void               **searchParams, 
		   LALInitSearchParams *initSearchParams 
		   ) 
{ 
  INT4               j,dimension,totalNumSeg,dataLength;  /* ,errorFlag=0;*/
  FCTSearchParams   *params;
  INT4               *stride;
  INT4   const       numParams=4;
  /********************* LAL thing **********************************/ 
  INITSTATUS(status, "LALInitSearch", FCTINITSEARCHC ); /* mandatory for  LAL*/
  ATTATCHSTATUSPTR (status);   

  
#ifdef FCT_SO_ATTACH_DDD
  {
   unsigned int doze = 10;
   pid_t myPID;
   myPID = getpid( );

   fprintf(stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
   fflush( stdout );
   
   sleep( doze );   
   fprintf(stdout,"pid %d awake\n", myPID );
   fflush( stdout );
   
  }
#endif
  

  if ( !(initSearchParams) )
  {
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL);
  }

  
  /************************************************************ 
		       relevant FCT part 

		       Read in the search parameters
		       initialize FCT   

		       searchMaster= 1: searchmaster; 0:slave; -1: wrappermaster

  **************************************************************/

  *searchParams = params=LALMalloc (sizeof( FCTSearchParams ));  

  if ( ! params )
  {
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }
  
  params->searchMaster=initSearchParams->nodeClass;
  params->firstTime= 1; 
  params->numSlaves =NULL;

  /*  check the validity of the input */
  if ( ! initSearchParams->argv )
  { 
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL ); 
  }

  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) || 
       atoi(initSearchParams->argv[1]) <=0 ||
       atoi(initSearchParams->argv[2]) <=0 || 
       ((initSearchParams->argc -3) != atoi(initSearchParams->argv[2])*4))
  { 
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }

 
  

  dimension=atoi(initSearchParams->argv[2])+1; /* number including dim 0 */
  params->startIndices      = LALMalloc(sizeof(INT4)*(dimension));
  params->endIndices        = LALMalloc(sizeof(INT4)*(dimension));
  params->indicesLength      = LALMalloc(sizeof(INT4)*(dimension));
  params->segLengthPerRun        = LALMalloc(sizeof(INT4)*(dimension));
  params->numSegPerDim         = LALMalloc(sizeof(INT4)*(dimension));
  stride=LALMalloc(sizeof(INT4)*(dimension));

  totalNumSeg=1;
  for (j=1; j<dimension; j++)
  {

    params->startIndices[j] =atoi(initSearchParams->argv[3+numParams*(j-1)]);
    params->endIndices[j] =atoi(initSearchParams->argv[3+numParams*(j-1)+1]);
    params->indicesLength[j]=(params->endIndices[j]-params->startIndices[j]+1);
    params->segLengthPerRun[j]=atoi(initSearchParams->argv[3+numParams*(j-1)+2]);
    stride[j]=atoi(initSearchParams->argv[3+numParams*(j-1)+3]);
    params->numSegPerDim[j]=params->indicesLength[j]/params->segLengthPerRun[j];
    totalNumSeg=totalNumSeg*params->numSegPerDim[j];
    if (params->startIndices[j] <0 || params->endIndices[j] <0 || 
	params->segLengthPerRun[j] <0 ||stride[j] <0 ||  
	params->startIndices[j] > params->endIndices[j] || 
	params->startIndices[j]+params->segLengthPerRun[j]-1>params->endIndices[j] || 
	stride[j] > params->segLengthPerRun[j])
    { 
      ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
    } 
  }

  /*  check that the dutyCycle >10, not yet  */

  params->totalNumSeg=totalNumSeg;
  params->segmentCompleted=0;

#ifdef FCT_SO_DEBUG
  printf("start/end_indices are %i, %i\n",params->startIndices[1],params->endIndices[1] );
  fprintf (stdout,"totalNumSeg at j=%i are %i\n", j, totalNumSeg);
  fflush( stdout );
#endif


  /*  if slave  */


  if ( ! params->searchMaster )
  {
    

    FCTInStrucType    fctInStrus;
    LALCreateFCTPlanInput planIn;
    LALFCTSetUnitsInput setUnitsIn; 

    params->fctInParams=(FCTInParamType *) LALMalloc (sizeof(FCTInParamType)); 
    if ( !params->fctInParams)
    {
      LALFree (searchParams); *searchParams = NULL;
      ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
    }

    dataLength=atoi(initSearchParams->argv[1]);
      
    params->fctInParams->dataLength=dataLength;
    params->fctInParams->dimension=dimension;
    /*  Check:  things specific to FCT; from argv? */
    planIn.data_length=dataLength/2;
    planIn.number_of_dimensions=dimension; /*  including dim 0  */
    planIn.dimension_0_stride=1; 

    for (j=0;j<dimension;j++)
    {
      planIn.phase_fn[j]=phaseFuncArray[j];
    }
    
    fctInStrus.planIn=planIn;

    /*  Define offset and as a constant there  */

    setUnitsIn.offset=0.0;
    setUnitsIn.delta=2.0/dataLength;

    fctInStrus.setUnitsIn=setUnitsIn;
    fctInStrus.maxFFTSeg=1;  /*  Check, as part of input? */
    params->fctInParams->cube=(LALFCTDataCube *)LALMalloc(sizeof(LALFCTDataCube));
    params->fctInParams->cube->stride[0]=planIn.dimension_0_stride;

    for (j=1; j<dimension; j++)
    {
      params->fctInParams->cube->stride[j]=stride[j];
    }
    
    params->fctInParams->numDataCube=1; /* Check, for now */
    params->fctInParams->cube->start_locations[0]=0;/* due to some bizarre thing in fct.c */
    params->fctInParams->cube->end_locations[0]=dataLength/2;

#ifdef FCT_SO_DEBUG
    fprintf(stdout,"data_Length =%i\n",dataLength);
    fflush( stdout );
#endif

    params->fctInParams->fctInput=NULL; /* necessary for LALCCreateVector() */
    LALCCreateVector(status->statusPtr, &(params->fctInParams->fctInput),dataLength);
    CHECKSTATUSPTR (status);
  
    params->fctInParams->fctPlan=NULL;
    LALMyFCTInitPlan(status->statusPtr, &fctInStrus, &(params->fctInParams->fctPlan));
    CHECKSTATUSPTR (status);
    
    
    if(!params->fctInParams->fctPlan)
     {
      ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL);
    }
    
  }/* endif slave  */

  /******************* LAL thing again ************/
  LALFree(stride);
#ifdef FCT_SO_DEBUG
  printf("leaving initSearch()\n");
#endif

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
} 


