/*----------------------------------------------------------------------- 
 * 
 * File Name: EPExch.c
 *
 * Author: 
 * 
 * Revision: $Id: EPExch.c,v 1.5 2001/05/15 22:18:07 patrick Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include <lal/AVFactories.h>
#include <lal/Comm.h>
#include "EPExch.h"

NRCSID (EPEXCHC, "$Id: EPExch.c,v 1.5 2001/05/15 22:18:07 patrick Exp $");

void
LALExchangeEPEvent (
    LALStatus        *status,
    TFTile           *event,
    ExchParams       *exchParams
    )
{
  CHARVector box; /* a box to hold some bytes of data */

  INITSTATUS (status, "LALExchangeEPEvent", EPEXCHC);
  ATTATCHSTATUSPTR (status);

  /* only do a minimal check to see if arguments are somewhat reasonable */
  ASSERT (exchParams, status, EPEXCH_ENULL, EPEXCH_MSGENULL);
  ASSERT (event, status, EPEXCH_ENULL, EPEXCH_MSGENULL);

  /* stuff the event into a box */
  box.length = sizeof (TFTile);
  box.data   = (CHAR *) event;

  if (exchParams->send)  /* I am sending */
  {
    INT4 dest = exchParams->partnerProcNum;

    /* send box */
    LALMPISendCHARVector (status->statusPtr, &box, dest, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
  }
  else /* I am receiving */
  {
    INT4 source = exchParams->partnerProcNum;

    /* receive box */
    LALMPIRecvCHARVector (status->statusPtr, &box, source, exchParams->mpiComm);
    CHECKSTATUSPTR (status);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


void
LALExchangeEPEventList (
    LALStatus     *status,
    TFTile       **eventHead,
    INT4           numEvents,
    ExchParams    *exchParams
    )
{
  INT4      i;
  TFTile   *eventCurrent = NULL;
  TFTile   *prev         = NULL;

  INITSTATUS (status, "LALExchangeEPEvent", EPEXCHC);
  ATTATCHSTATUSPTR (status);

  if (exchParams->send)  /* I am sending */
  {

    /* check that we have a bank to send */
    ASSERT (*eventHead, status, EPEXCH_ENULL, EPEXCH_MSGENULL);

    /* exchange the event list */
    for ( i=0, eventCurrent = *eventHead;
        i<numEvents && eventCurrent != NULL;
        eventCurrent = eventCurrent->nextTile, ++i
        )
    {
      LALExchangeEPEvent (status->statusPtr, eventCurrent, exchParams);
      CHECKSTATUSPTR (status);
    }

  }
  else /* I am receiving */
  {

    /* check that this is a new list */
    ASSERT (!*eventHead, status, EPEXCH_ENNUL, EPEXCH_MSGENNUL);

    i=0;
    /* recieve the template bank */
    do
    {
      /* create memory for the template */
      eventCurrent = (TFTile *) LALMalloc (sizeof(TFTile));

      /* make a note of the first node in the list to return */
      if ( *eventHead == NULL ) *eventHead = eventCurrent;

      /* recieve the template */
      LALExchangeEPEvent (status->statusPtr, eventCurrent, exchParams);
      CHECKSTATUSPTR (status);

      /* point the previous node to this node */
      if ( prev != NULL ) prev->nextTile = eventCurrent;
      prev = eventCurrent;

      /* increment pointer */
      ++i;
    } while ( eventCurrent->nextTile != NULL && i<numEvents);

    eventCurrent->nextTile = NULL;

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


