/***************************** <lalVerbatim file="StochasticBarApplySearchCV">
Author: Anderson, W.G., Heng, I.S., Romano, J.D., and Whelan, J.T.
$Id: StochasticBarApplySearch.c,v 1.8 2001/09/02 05:57:29 heng Exp $
 **************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBarApplySearchC">
  \subsection{Module \texttt{StochasticBarApplySearch.c}}
  \label{s:StochasticBarApplySearch.c}

  The LAL-wrapper interface function for ...

  \subsection*{Description}

  \subsection*{Algorithms}

  \subsection*{Uses}

  \subsection*{Notes}

  \vfill{\footnotesize\input{StochasticBarApplySearchCV}}

 *************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "StochasticBarApplySearch.h"

NRCSID (STOCHASTICAPPLYSEARCHC, "$Id: StochasticBarApplySearch.c,v 1.8 2001/09/02 05:57:29 heng Exp $");

void
LALApplySearch(
      LALStatus             *status,
      LALSearchOutput       *output,
      LALSearchInput        *input,
      LALApplySearchParams  *applyParams
              )
{

   INT4                                myRank;

   COMPLEX8FrequencySeries             fseries1;
   COMPLEX8FrequencySeries             fseries2;
   COMPLEX8FrequencySeries             ccSpectrum;
   COMPLEX8FrequencySeries             ccSpectrumOut;
   StochasticCrossCorrelationInput     ccInput;
   FrequencySamplingParams             cgParams;
   gpsTimeInterval                     timeInterval;
   UINT8                               segLengthNanoSeconds;
   UINT8                               segStartNanoSeconds;
   UINT8                               segStopNanoSeconds;

   StochasticSearchParams*             params = NULL;
   StochasticDataSegment*              dummySegment = NULL;

//   COMPLEX8WithUnits	*testoutput;

   INITSTATUS (status, "LALApplySearch", STOCHASTICAPPLYSEARCHC);
   ATTATCHSTATUSPTR (status);


   /* 
    *
    * check output and parameters 
    * 
    */

   ASSERT (output, 
         status, 
         STOCHASTICAPPLYSEARCHH_ENULL, 
         STOCHASTICAPPLYSEARCHH_MSGENULL);

   ASSERT (!(output->result), 
         status, 
         STOCHASTICAPPLYSEARCHH_ENNUL, 
         STOCHASTICAPPLYSEARCHH_MSGENNUL);

   ASSERT (applyParams, 
         status, 
         STOCHASTICAPPLYSEARCHH_ENULL, 
         STOCHASTICAPPLYSEARCHH_MSGENULL);

   ASSERT (applyParams->mpiParams, 
         status, 
         STOCHASTICAPPLYSEARCHH_ENULL, 
         STOCHASTICAPPLYSEARCHH_MSGENULL);

   ASSERT (applyParams->searchParams, 
         status, 
         STOCHASTICAPPLYSEARCHH_ENULL, 
         STOCHASTICAPPLYSEARCHH_MSGENULL);


   /* cast the parameter structure */ 

   params = (StochasticSearchParams *) applyParams->searchParams;

//printf("App1. status->statusCode %i\n",status->statusCode);
   /* 
    *
    * perform the search
    *
    */

   /* 
    * 
    * NOTE: WE DO THE ANALYSIS ON **ONLY** THE SEARCH MASTER SINCE
    * WE CONSIDER THE SIMPLE CASE OF A SEARCH FOR A STOCHASTIC
    * BACKGROUND SIGNAL WITH OMEGA_GW=CONST.  
    * SEARCHES FOR STOCHASTIC SIGNALS HAVING DIFFERENT SPECTRA CAN 
    * BE DONE ON MULTIPLE SEARCH SLAVES.
    * THIS IS WORK FOR THE FUTURE!!
    * 
    */   

   /* if not search master, abort */
   if (!(params->searchMaster))
   {
      ABORT (status, 
            STOCHASTICAPPLYSEARCHH_EMSTR, 
            STOCHASTICAPPLYSEARCHH_MSGEMSTR);
   }

   /* determine rank and check that search master is running on node 0 */
   /* :TODO: is this correct?? */
   MPI_Comm_rank (*(applyParams->mpiParams->comm), &myRank);

   if (myRank != 0)
   {
      ABORT (status, 
            STOCHASTICAPPLYSEARCHH_ERANK, 
            STOCHASTICAPPLYSEARCHH_MSGERANK);
   }

//printf("App2. status->statusCode %i\n",status->statusCode);

   if (params->currentSegment < params->initParams->numSegments)
   {

      /* we still have data to analyze, so carry on with search */


      /* 
       *
       * allocate temporary memory for the frequency data 
       *
       */

      fseries1.data = NULL;
      LALCCreateVector (status->statusPtr, 
			&(fseries1.data), 
			2*(params->initParams->numPoints)-1);
      CHECKSTATUSPTR (status);

      fseries2.data = NULL;
      LALCCreateVector (status->statusPtr, 
			&(fseries2.data), 
			2*(params->initParams->numPoints)-1);
      CHECKSTATUSPTR (status);

      ccSpectrum.data = NULL;
      LALCCreateVector (status->statusPtr, 
            &(ccSpectrum.data), 
            params->initParams->numFrequencies);
      CHECKSTATUSPTR (status);

      ccSpectrumOut.data = NULL;
      LALCCreateVector (status->statusPtr, 
            &(ccSpectrumOut.data), 
            params->initParams->outputNumFreq);
      CHECKSTATUSPTR (status);

//printf("App3. status->statusCode %i\n",status->statusCode);

      /* point dummySegment to the current segment to analyze */
      dummySegment = params->stochasticSegVec->data + params->currentSegment;

      // printf("%d \n", params->currentSegment);

      /* set fields of time interval */
      timeInterval.numberSamples = (UINT8) (params->initParams->numPoints);
      timeInterval.startSec = (UINT8) (dummySegment->data1->epoch.gpsSeconds);
      timeInterval.startNan 
	= (UINT8) (dummySegment->data1->epoch.gpsNanoSeconds);
      timeInterval.timeStepSize = (REAL8) (dummySegment->data1->deltaT);
      segStartNanoSeconds = 1000000000L * (INT8) (timeInterval.startSec);
      segStartNanoSeconds += timeInterval.startNan;
      segLengthNanoSeconds = (INT8) (1.0e9 * timeInterval.numberSamples * 
            timeInterval.timeStepSize);
      segStopNanoSeconds = segStartNanoSeconds + segLengthNanoSeconds;
      timeInterval.stopSec = (UINT8) (segStopNanoSeconds / 1000000000L);
      timeInterval.stopNan = (UINT8) (segStopNanoSeconds % 1000000000L);
      // printf("%d %d \n", timeInterval.startSec, timeInterval.startNan);


      /* 
       *
       * zero-pad and FFT the time-series data data 
       *
       */

//printf("params->initParams->numPoints is %d \n",  params->initParams->numPoints);

/* f0 manually set (for now) to middle of spectra in complex time series */

      dummySegment->data1->f0 = 907.0;
      dummySegment->data2->f0 = 907.0;
//printf("dummySegment->data1->f0 is %f \n",dummySegment->data1->f0);
//printf("dummySegment->data1->deltaT is %f \n",dummySegment->data1->deltaT);

      LALCZeroPadAndFFT (status->statusPtr,
            &fseries1,
            dummySegment->data1,
            params->fftPlan);
      CHECKSTATUSPTR (status);

      LALCZeroPadAndFFT (status->statusPtr,
            &fseries2,
            dummySegment->data2,
            params->fftPlan);
      CHECKSTATUSPTR (status);

//printf("App4. status->statusCode %i\n",status->statusCode);
      /* 
       *
       * build stochastic cross-correlation input structure 
       *
       */

      ccInput.hBarTildeOne  = &fseries1;
      ccInput.hBarTildeTwo  = &fseries2;
      ccInput.optimalFilter = params->optimalFilter;

//printf("params->initParams->numPoints is %d \n",  params->initParams->numPoints);

//printf("ccInput.hBarTildeOne->f0 is %f\n",ccInput.hBarTildeOne->f0);
//printf("ccInput.hBarTildeOne->deltaF is %f\n",ccInput.hBarTildeOne->deltaF);
      /* 
       *
       * calculate the cross-correlation spectrum 
       *
       */
      LALStochasticCrossCorrelationSpectrum (status->statusPtr,
					     &ccSpectrum,
					     &ccInput,
					     params->initParams->epochsMatch);

//	LALStochasticHeterodynedCrossCorrelationStatistic(status->statusPtr,
//					     testoutput,
//					     &ccInput,
//					     params->initParams->epochsMatch);
      CHECKSTATUSPTR (status);

      cgParams.length = params->initParams->outputNumFreq;
      cgParams.f0     = params->initParams->outputF0;
      cgParams.deltaF = params->initParams->outputDeltaF;

      /* Coarse-grain the output to the desired resolution */
      TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					  &ccSpectrumOut,
					  &ccSpectrum,
					  &cgParams),
	   status);

//printf("App5. status->statusCode %i\n",status->statusCode);

      /* 
       *
       * build output for metadata database or trend frames 
       *
       */
      StochasticBarBuildOutput(status->statusPtr,
            output,
           &ccSpectrumOut,
            timeInterval);
      CHECKSTATUSPTR (status);

      /* 
       *
       * remove temporary memory for the frequency data 
       *
       */

/* temporarily not check status. ccSpectrumOut.data refuses to be
   destroyed so ignore error for now. This has to be fixed because 
   it cauces a memory leak. The problem is in StochasticBuildOutput
   but it has not been tracked down yet. */

      LALCDestroyVector (status->statusPtr, &(ccSpectrumOut.data));
//printf("App6. status->statusCode %i\n",status->statusCode);
//      CHECKSTATUSPTR (status);

      LALCDestroyVector (status->statusPtr, &(ccSpectrum.data));
      CHECKSTATUSPTR (status);

      LALCDestroyVector (status->statusPtr, &(fseries2.data));
      CHECKSTATUSPTR (status);

      LALCDestroyVector (status->statusPtr, &(fseries1.data));
      CHECKSTATUSPTR (status);

      /*
       *  
       * compute fraction of search remaining for wrapper and set 
       * output not finished 
       * 
       * :NOTE: this is what duncan did for simple.so even though
       * it would seem that i should increment to the pointer
       * to the current segment before computing fracRemaining and
       * before determining how to set notFinished.
       *
       */  

      output->fracRemaining = 1.0 - 
         (REAL4)(params->currentSegment)/ 
         (REAL4)(params->initParams->numSegments);

      output->notFinished = STOCHASTICH_TRUE;


      /* increment pointer to current segment */
      (params->currentSegment)++;

   }
   else
   {
      /* we are finished with the search */

      output->fracRemaining = 0;
      output->notFinished   = STOCHASTICH_FALSE;
   }


   /* normal exit */ 
   DETATCHSTATUSPTR (status);
   RETURN (status);
}

