/***************************** <lalVerbatim file="StochasticBarInitSearchCV">
Author: Anderson, W.G., Heng, I.S., Romano, J.D., and Whelan, J.T.
$Id: StochasticBarInitSearch.c,v 1.6 2001/09/02 05:59:02 heng Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBarInitSearchC">
\subsection{Module \texttt{StochasticBarInitSearch.c}}
\label{s:StochasticBarInitSearch.c}

The LAL-wrapper interface function for ...

\subsubsection*{Prototypes}
\input{StochasticBarInitSearchCP}
\index{\texttt{LALInitSearch()}}

\subsection*{Description}

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticBarInitSearchCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/DetectorSite.h>
#include "StochasticBarData.h"
#include "StochasticBarInitSearch.h"

NRCSID (STOCHASTICINITSEARCHC, "$Id: StochasticBarInitSearch.c,v 1.6 2001/09/02 05:59:02 heng Exp $");

/************************ <lalVerbatim file="StochasticBarInitSearchCP"> */
void LALInitSearch(
    LALStatus*              status,
    void**                  searchParams,
    LALInitSearchParams*    initSearchParams
)
/*********************************************** </lalVerbatim> */
{
    StochasticSearchParams *params = NULL;
    UINT4 stringLength;

    REAL8 fMin, fMinOut;
    
    INITSTATUS (status, "LALInitSearch", STOCHASTICINITSEARCHC);
    ATTATCHSTATUSPTR (status);

//    lalDebugLevel = LALALLDBG;
    
    /* sleep to give me time to attach ddd */
    #ifdef STOCHASTIC_SO_ATTACH_DDD
    {
        unsigned int doze = 10;
        pid_t myPID;
        myPID = getpid();
        fprintf (stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
        fflush (stdout);
        sleep (doze);
        fprintf (stdout, "pid %d awake\n", myPID);
        fflush (stdout);
    }
    #endif

    /* Check architecture to make sure BLOb we write is sensible */

    if ( sizeof(UCHAR) != 1 || sizeof(COMPLEX8) != 8 )
    {
      ABORT( status, STOCHASTICINITSEARCHH_ESIZE, 
	     STOCHASTICINITSEARCHH_MSGEDTSZ );
    }

    /*
     *  
     * check input, output, and params
     *  
     */
    
    ASSERT (searchParams, 
            status,
            STOCHASTICINITSEARCHH_ENULL, 
            STOCHASTICINITSEARCHH_MSGENULL );

    ASSERT (!(*searchParams), 
            status,
            STOCHASTICINITSEARCHH_ENNUL, 
            STOCHASTICINITSEARCHH_MSGENNUL );

    ASSERT (initSearchParams, 
            status,
            STOCHASTICINITSEARCHH_ENULL, 
            STOCHASTICINITSEARCHH_MSGENULL );

    /* check command line arguments */
 
    if (!(initSearchParams->argv)) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENULL, 
               STOCHASTICINITSEARCHH_MSGENULL);
    }
    if (initSearchParams->argc != STOCHASTICINITSEARCHH_NPARAMS) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EARGS, 
               STOCHASTICINITSEARCHH_MSGEARGS ); 
    }
    if (strcmp (initSearchParams->argv[0], "-filterparams" )) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EARG0, 
               STOCHASTICINITSEARCHH_MSGEARG0);
    }
    if (atoi (initSearchParams->argv[1]) <= 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENSEG, 
               STOCHASTICINITSEARCHH_MSGENSEG);
    }
    if (atoi (initSearchParams->argv[2]) <= 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENPTS, 
               STOCHASTICINITSEARCHH_MSGENPTS);
    }
    if (atoi( initSearchParams->argv[3] ) < 1 ){
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENFRQ, 
               STOCHASTICINITSEARCHH_MSGENFRQ);
    }
    if (atof (initSearchParams->argv[4]) < 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EF0, 
               STOCHASTICINITSEARCHH_MSGEF0);
    }
    if (atof (initSearchParams->argv[5]) <= 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EDELF, 
               STOCHASTICINITSEARCHH_MSGEDELF);
    }
    if (atoi( initSearchParams->argv[6] ) < 1 ){
        ABORT (status, 
               STOCHASTICINITSEARCHH_ENFRQ, 
               STOCHASTICINITSEARCHH_MSGENFRQ);
    }
    if (atof (initSearchParams->argv[7]) < 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EF0, 
               STOCHASTICINITSEARCHH_MSGEF0);
    }
    if (atof (initSearchParams->argv[8]) <= 0) {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EDELF, 
               STOCHASTICINITSEARCHH_MSGEDELF);
    }
    if (atoi( initSearchParams->argv[14] ) >= LALNumCachedDetectors ){
        ABORT (status, 
               STOCHASTICINITSEARCHH_ESTID, 
               STOCHASTICINITSEARCHH_MSGESTID);
    }    
    if (atoi( initSearchParams->argv[15] ) >= LALNumCachedDetectors ){
        ABORT (status, 
               STOCHASTICINITSEARCHH_ESTID, 
               STOCHASTICINITSEARCHH_MSGESTID);
    }    

    /* 
     *
     * allocate memory for parameter structure 
     *
     */
    
    *searchParams = params = 
        LALMalloc (sizeof (StochasticSearchParams)); 
    if (!params)
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }

    params->initParams = (StochasticInitParams *)
        LALMalloc (sizeof (StochasticInitParams));
    if (!(params->initParams))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }

        
    /*
     *
     *  assign the initial parameter values 
     *
     */

    params->initParams->numSegments     
        = (UINT4) atoi(initSearchParams->argv[1]);
    params->initParams->numPoints       
        = (UINT4) atoi(initSearchParams->argv[2]); 
    params->initParams->numFrequencies 
        = (UINT4) atoi(initSearchParams->argv[3]);
    params->initParams->f0     = atof (initSearchParams->argv[4]);
    params->initParams->deltaF = atof (initSearchParams->argv[5]);
    params->initParams->outputNumFreq
        = (UINT4) atoi(initSearchParams->argv[6]);
    params->initParams->outputF0     = atof (initSearchParams->argv[7]);
    params->initParams->outputDeltaF = atof (initSearchParams->argv[8]);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[9]) + 1;
    params->initParams->dmro1name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->dmro1name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->dmro1name,initSearchParams->argv[9],
	   stringLength);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[10]) + 1;
    params->initParams->dmro2name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->dmro2name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->dmro2name,initSearchParams->argv[10],
	   stringLength);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[11]) + 1;
    params->initParams->response1name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->response1name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->response1name,initSearchParams->argv[11],
	   stringLength);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[12]) + 1;
    params->initParams->response2name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->response2name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->response2name,initSearchParams->argv[12],
	   stringLength);

    params->initParams->epochsMatch
      = (BOOLEAN) atoi(initSearchParams->argv[13]);
    params->initParams->siteID1
      = (UINT4) atoi(initSearchParams->argv[14]);
    params->initParams->siteID2
      = (UINT4) atoi(initSearchParams->argv[15]);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[16]) + 1;
    params->initParams->spectrum1name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->spectrum1name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->spectrum1name,initSearchParams->argv[16],
	   stringLength);

    /* note that this includes the trailing '\0' */
    stringLength = strlen(initSearchParams->argv[17]) + 1;
    params->initParams->spectrum2name
      = (CHAR *) LALMalloc(stringLength * sizeof(CHAR));
    if (!(params->initParams->spectrum2name))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    memcpy(params->initParams->spectrum2name,initSearchParams->argv[17],
	   stringLength);

    params->initParams->allegroLongitude
      = (REAL8) atof(initSearchParams->argv[18]);
    params->initParams->allegroLatitude
      = (REAL8) atof(initSearchParams->argv[19]);
    params->initParams->allegroAzimuth
      = (REAL4) atof(initSearchParams->argv[20]);
 
    /* Check consistency of arguments */
 
    if (params->initParams->outputDeltaF < params->initParams->deltaF)
    {
      ABORT (status,
             STOCHASTICINITSEARCHH_EFRES,
             STOCHASTICINITSEARCHH_MSGEFRES);
    }
 
    fMin = params->initParams->f0
      || params->initParams->f0 - ( params->initParams->deltaF / 2 );
    fMinOut = params->initParams->outputF0
      || params->initParams->outputF0
      - ( params->initParams->outputDeltaF / 2 );
 
    if (fMinOut < fMin)
    {
      ABORT (status,
             STOCHASTICINITSEARCHH_EFRNG,
             STOCHASTICINITSEARCHH_MSGEFRNG);
    }
 
    if ( params->initParams->outputF0
           + ( (REAL8) params->initParams->outputNumFreq - 0.5 )
             * params->initParams->outputDeltaF
         > params->initParams->f0
           + ( (REAL8) params->initParams->numFrequencies - 0.5 )
             * params->initParams->deltaF )
    {
      ABORT (status,
             STOCHASTICINITSEARCHH_EFRNG,
             STOCHASTICINITSEARCHH_MSGEFRNG);
    }

    /* :TODO: check that searchMaster and haveData are set properly */
    /* I think these are right! WGA */
    params->searchMaster    = initSearchParams->nodeClass; 
    params->haveData        = 0;
    params->currentSegment  = 0;
    
    
    /* 
     *
     * allocate memory for non-trivial parameter fields 
     *
     */
   
         
    /* optimal filter */
    params->optimalFilter = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));
    if (!(params->optimalFilter))
    {
        ABORT (status, 
               STOCHASTICINITSEARCHH_EALOC, 
               STOCHASTICINITSEARCHH_MSGEALOC);
    }
    
    params->optimalFilter->data = NULL;
    LALCCreateVector (status->statusPtr,
                      &(params->optimalFilter->data),
                      params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* fftPlan */
    params->fftPlan = NULL;
    LALCreateForwardComplexFFTPlan(status->statusPtr,
				&(params->fftPlan),
				2*(params->initParams->numPoints) - 1,
				STOCHASTICH_TRUE);
    CHECKSTATUSPTR (status);
    
    /* data segment vector */     
    params->stochasticSegVec = NULL;
    LALCreateStochasticDataSegmentVector (status->statusPtr, 
                                          &(params->stochasticSegVec), 
                                          params->initParams);
    CHECKSTATUSPTR (status);

    
    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

