/********************************** <lalVerbatim file="StochasticDataHV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticData.h,v 1.7 2001/08/26 17:19:24 whelan Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticDataH">
\section{Header \texttt{StochasticData.h}}
\label{s:StochasticData.h}

Header file for ...

\subsection*{Synopsis}
\begin{verbatim}
#include "StochasticData.h"
\end{verbatim}

\noindent
This header ...

\subsection*{Error conditions}
\input{StochasticDataHErrTab}

\subsection*{Structures}

*************************************************** </lalLaTeX> */

#ifndef _STOCHASTICDATA_H
#define _STOCHASTICDATA_H

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/LALRCSID.h>

#ifdef  __cplusplus   /* C++ protection. */
extern "C" {
#endif

NRCSID (STOCHASTICDATAH, "$Id: StochasticData.h,v 1.7 2001/08/26 17:19:24 whelan Exp $");

/******** <lalErrTable file="StochasticDataHErrTab"> ********/
#define STOCHASTICDATAH_ENNUL       1 
#define STOCHASTICDATAH_ENULL       2
#define STOCHASTICDATAH_ENSEG       3
#define STOCHASTICDATAH_ENPTS       4
#define STOCHASTICDATAH_ENFRQ       5
#define STOCHASTICDATAH_EALOC       6
#define STOCHASTICDATAH_ESTID       7

#define STOCHASTICDATAH_MSGENNUL    "Non-null pointer"
#define STOCHASTICDATAH_MSGENULL    "Null pointer"
#define STOCHASTICDATAH_MSGENSEG    "Number of data segments is zero or negative"
#define STOCHASTICDATAH_MSGENPTS    "Number of points in a data segment is zero or negative"
#define STOCHASTICDATAH_MSGENFRQ    "Number of frequencies is zero or negative"
#define STOCHASTICDATAH_MSGEALOC    "Memory allocation error"   
#define STOCHASTICDATAH_MSGESTID    "Detector Site ID out of range"
/******** </lalErrTable> ********/

/******************************* <lalLaTeX file="StochasticDataH">

\subsubsection*{\texttt{struct StochasticInitParams}}
\index{\texttt{StochasticInitParams}}

\noindent
The fields are:
\begin{description}
\item[\texttt{UINT4 numSegments}] Number of data segments
\item[\texttt{UINT4 numPoints}] Number of points in a data segment
\item[\texttt{UINT4 numFrequencies}] Number of frequencies in optimal filter
\item[\texttt{REAL8 f0}] Start frequency for optimal filter
\item[\texttt{REAL8 deltaF}] Frequency spacing for optimal filter
\item[\texttt{UINT4 numFrequencies}] Number of frequencies in cross-correlation spectrum
\item[\texttt{REAL8 f0}] Start frequency for CC spectrum
\item[\texttt{REAL8 deltaF}] Frequency spacing for CC spectrum
\item[\texttt{CHAR *dmro1name}] Name of first data channel
\item[\texttt{CHAR *dmro2name}] Name of second data channel
\item[\texttt{CHAR *response1name}] Name of first response function
\item[\texttt{CHAR *response2name}] Name of second response function
\item[\texttt{UINT4 siteID1}] index of first detector in \texttt{lalCachedDetectors}
\item[\texttt{UINT4 siteID2}] index of second detector in \texttt{lalCachedDetectors}
\item[\texttt{CHAR *spectrum1name}] Name of first PSD
\item[\texttt{CHAR *spectrum2name}] Name of second PSD
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticInitParams
{
  UINT4                           numSegments;
  UINT4                           numPoints;
  UINT4                           numFrequencies;
  REAL8                           f0;
  REAL8                           deltaF;
  UINT4                           outputNumFreq;
  REAL8                           outputF0;
  REAL8                           outputDeltaF;
  CHAR                           *dmro1name;
  CHAR                           *dmro2name;
  CHAR                           *response1name;
  CHAR                           *response2name;
  BOOLEAN                         epochsMatch;
  UINT4                           siteID1;
  UINT4                           siteID2;
  CHAR                           *spectrum1name;
  CHAR                           *spectrum2name;
}
StochasticInitParams;

/******************************* <lalLaTeX file="StochasticDataH">

\subsubsection*{\texttt{struct StochasticDataSegment}}
\index{\texttt{StochasticDataSegment}}

\noindent
The fields are:
\begin{description}
\item[\texttt{REAL4TimeSeries *data1}]
\item[\texttt{REAL4TimeSeries *data2}]
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticDataSegment
{
    REAL4TimeSeries*                data1;
    REAL4TimeSeries*                data2;
}
StochasticDataSegment;

/******************************* <lalLaTeX file="StochasticDataH">

\subsubsection*{\texttt{struct StochasticDataSegmentVector}}
\index{\texttt{StochasticDataSegmentVector}}

\noindent
The fields are:
\begin{description}
\item[\texttt{UINT4 length}]
\item[\texttt{StochasticDataSegment *data}]
\end{description}

*************************************************** </lalLaTeX> */

typedef struct
tagStochasticDataSegmentVector
{
    UINT4                           length;
    StochasticDataSegment*          data;
}
StochasticDataSegmentVector;


void
LALCreateStochasticDataSegmentVector (
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector,
    StochasticInitParams*           params
);


void
LALDestroyStochasticDataSegmentVector (
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector
);


#ifdef  __cplusplus
}
#endif /* C++ protection. */

#endif /* _STOCHASTICDATA_H  */

/******************************* <lalLaTeX file="StochasticDataH">
  
\vfill{\footnotesize\input{StochasticDataHV}}

\newpage\input{StochasticDataC}

*************************************************** </lalLaTeX> */
